#include <thread>
#include <string>
#include <unistd.h>
#include <time.h>
#include <math.h>

// pro změření obecného časového intervalu
class Cas
{
protected:
	timespec ts_start;
	timespec ts_stop;
	clockid_t typ = CLOCK_REALTIME;

public:
	void start() noexcept
	{
		clock_gettime(this->typ, &this->ts_start);
	}

	void stop() noexcept
	{
		clock_gettime(this->typ, &this->ts_stop);
	}
	
	double interval() noexcept
	{
		return ts_stop.tv_sec - ts_start.tv_sec +
			(ts_stop.tv_nsec - ts_start.tv_nsec) / 1000000000.;
	}
};

// pro změření spotřebovaného procesorového času celého procesu
class CasCpuProcess : public Cas
{
public:
	CasCpuProcess() noexcept
	{
		this->typ = CLOCK_PROCESS_CPUTIME_ID;
	}
};

// pro změření spotřebovaného procesorového času jedntlivého vlákna
class CasCpuThread : public Cas
{
public:
	CasCpuThread() noexcept
	{
		this->typ = CLOCK_THREAD_CPUTIME_ID;
	}
};

bool je_prvocislo(unsigned int cislo) noexcept
{
	unsigned int mez = (unsigned int)sqrt(cislo);
	for (unsigned int i = 2; i <= mez; i++)
	{
		if (0 == cislo % i)
			return false;
	}
	return true;
}

void funkce_thread(size_t pocet, int vlakno) noexcept
{
	Cas cas;
	CasCpuThread cas_thread;
	cas.start();
	cas_thread.start();
	unsigned int pocet_prv = 0;
	for (size_t i = 0; i < pocet; i++)
	{
		if (je_prvocislo(i))
			pocet_prv++;
	}
	sleep(2); // 2 sekundy které nezaměstnají procesor
	cas_thread.stop();
	cas.stop();
	printf("thread: %d, čas: %.6f, čas cpu thread: %.6f (počet: %d)\n",
		vlakno, cas.interval(), cas_thread.interval(), (int)pocet_prv);
}

int main(int argc, const char** argv)
{
	#define POCET_T 4
	std::thread* threads[POCET_T];
	Cas cas;
	CasCpuProcess cas_process;
	cas.start();
	cas_process.start();
	for (size_t i = 0; i < POCET_T; i++)
	{
		threads[i] = new std::thread(funkce_thread, 10000000, i);
	}
	for (size_t i = 0; i < POCET_T; i++)
	{
		threads[i]->join();
	}
	for (size_t i = 0; i < POCET_T; i++)
	{
		delete threads[i];
	}
	cas.stop();
	cas_process.stop();
	printf("čas celkem: %.6f, čas cpu procesu: %.6f\n",
		cas.interval(), cas_process.interval());
	printf("----------\nhotovo\n");
	return EXIT_SUCCESS;
}
